@@ -4,8 +4,6 @@ import dask
 import dask.array as da
 import dask.dataframe as dd
 import numpy as np
-import pandas as pd
-import scipy.sparse as sp
 import sklearn.decomposition
 from dask import compute
 from sklearn.utils.extmath import fast_logdet
@@ -201,9 +199,11 @@ class PCA(sklearn.decomposition.PCA):
         if not dask.is_dask_collection(X):
             raise TypeError(_TYPE_MSG.format(type(X)))
         self._fit(X)
+        self.n_features_in_ = X.shape[1]
         return self
 
-    def _fit(self, X):
+    def _get_solver(self, X, n_components):
+        n_samples, n_features = X.shape
         solvers = {"full", "auto", "tsqr", "randomized"}
         solver = self.svd_solver
 
@@ -212,22 +212,6 @@ class PCA(sklearn.decomposition.PCA):
                 "Invalid solver '{}'. Must be one of {}".format(solver, solvers)
             )
 
-        if isinstance(X, dd.DataFrame):
-            X = X.values
-
-        # Handle n_components==None
-        if self.n_components is None:
-            # TODO: handle nan shapes
-            n_components = min(X.shape)
-        elif 0 < self.n_components < 1:
-            raise NotImplementedError(
-                "Fractional 'n_components' is not " "currently supported"
-            )
-        else:
-            n_components = self.n_components
-
-        n_samples, n_features = X.shape
-
         if solver == "auto":
             # Small problem, just call full PCA
             if not _known_shape(X.shape):
@@ -263,6 +247,25 @@ class PCA(sklearn.decomposition.PCA):
                 )
             )
             raise ValueError(msg)
+        return solver
+
+    def _fit(self, X):
+        if isinstance(X, dd.DataFrame):
+            X = X.values
+
+        # Handle n_components==None
+        if self.n_components is None:
+            # TODO: handle nan shapes
+            n_components = min(X.shape)
+        elif 0 < self.n_components < 1:
+            raise NotImplementedError(
+                "Fractional 'n_components' is not " "currently supported"
+            )
+        else:
+            n_components = self.n_components
+
+        n_samples, n_features = X.shape
+        solver = self._get_solver(X, n_components)
 
         self.mean_ = X.mean(0)
         X -= self.mean_
