@@ -4,7 +4,7 @@ These estimators will operate in parallel. Their scalability depends
 on the underlying estimators being used.
 """
 from ._hyperband import HyperbandSearchCV
-from ._incremental import IncrementalSearchCV
+from ._incremental import IncrementalSearchCV, InverseDecaySearchCV
 from ._search import GridSearchCV, RandomizedSearchCV, check_cv, compute_n_splits
 from ._split import KFold, ShuffleSplit, train_test_split
 from ._successive_halving import SuccessiveHalvingSearchCV
@@ -20,4 +20,5 @@ __all__ = [
     "IncrementalSearchCV",
     "HyperbandSearchCV",
     "SuccessiveHalvingSearchCV",
+    "InverseDecaySearchCV",
 ]
