@@ -1,33 +1,55 @@
 from __future__ import division
 
+import itertools
+import logging
 import operator
+import sys
 from collections import defaultdict, namedtuple
 from copy import deepcopy
 from time import time
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from warnings import warn
 
 import dask
 import dask.array as da
+import dask.dataframe as dd
 import numpy as np
 import scipy.stats
 import toolz
 from dask.distributed import Future, default_client, futures_of, wait
 from distributed.utils import log_errors
-from sklearn.base import clone
+from sklearn.base import BaseEstimator, clone
+from sklearn.linear_model import SGDClassifier
 from sklearn.metrics import check_scoring
 from sklearn.model_selection import ParameterGrid, ParameterSampler
 from sklearn.utils import check_random_state
 from sklearn.utils.metaestimators import if_delegate_has_method
-from tornado import gen
 
-from .._compat import check_is_fitted
-from ..utils import check_array
+from .._compat import check_is_fitted, dummy_context
+from .._typing import ArrayLike, Int
+from .._utils import LoggingContext
 from ..wrappers import ParallelPostFit
 from ._split import train_test_split
 
+logger = logging.getLogger("dask_ml.model_selection")
+
+no_default = object()
+
 Results = namedtuple("Results", ["info", "models", "history", "best"])
+Params = Dict[str, Any]
+Meta = Dict[str, Any]  # really Dict[str, Union[int, float, str, Params]]
+Model = Union[BaseEstimator, SGDClassifier]
+History = List[Meta]
+Info = Dict[Int, History]
+Instructions = Dict[Int, Int]
 
 
-def _partial_fit(model_and_meta, X, y, fit_params):
+def _partial_fit(
+    model_and_meta: Tuple[Model, Meta],
+    X: ArrayLike,
+    y: ArrayLike,
+    fit_params: Dict[str, Any],
+) -> Tuple[Model, Meta]:
     """
     Call partial_fit on a classifiers with training data X and y
 
@@ -86,7 +108,12 @@ def _partial_fit(model_and_meta, X, y, fit_params):
         return model, meta
 
 
-def _score(model_and_meta, X, y, scorer):
+def _score(
+    model_and_meta: Tuple[Model, Meta],
+    X: ArrayLike,
+    y: ArrayLike,
+    scorer: Optional[Callable[[Model, ArrayLike, ArrayLike], float]],
+) -> Meta:
     start = time()
     model, meta = model_and_meta
     if scorer:
@@ -99,60 +126,87 @@ def _score(model_and_meta, X, y, scorer):
     return meta
 
 
-def _create_model(model, ident, **params):
+def _create_model(model: Model, ident: Int, **params: Params) -> Tuple[Model, Meta]:
     """ Create a model by cloning and then setting params """
-    with log_errors(pdb=True):
+    with log_errors():
         model = clone(model).set_params(**params)
         return model, {"model_id": ident, "params": params, "partial_fit_calls": 0}
 
 
-@gen.coroutine
-def _fit(
-    model,
-    params,
-    X_train,
-    y_train,
-    X_test,
-    y_test,
-    additional_calls,
-    fit_params=None,
-    scorer=None,
+async def _fit(
+    model: Model,
+    params: Union[List[Params], ParameterSampler, ParameterGrid],
+    X_train: ArrayLike,
+    y_train: ArrayLike,
+    X_test: ArrayLike,
+    y_test: ArrayLike,
+    additional_calls: Callable[[Info], Instructions],
+    fit_params: Dict[str, Any] = None,
+    scorer: Callable[[Model, ArrayLike, ArrayLike], float] = None,
     random_state=None,
-):
+    verbose: Union[bool, Int, float] = False,
+    prefix: str = "",
+) -> Results:
+    if isinstance(verbose, bool):
+        # Always log (other loggers might configured differently)
+        verbose = 1.0
+    if not 0 <= verbose <= 1:
+        raise ValueError(
+            "verbose={} does not satisfy 0 <= verbose <= 1".format(verbose)
+        )
+    log_delay = int(1 / float(verbose)) if verbose > 0 else 0
+
     original_model = model
     fit_params = fit_params or {}
     client = default_client()
     rng = check_random_state(random_state)
 
-    info = {}
-    models = {}
-    scores = {}
+    info: Dict[int, History] = {}
+    models: Dict[int, Tuple[Model, Meta]] = {}
+    scores: Dict[int, Meta] = {}
 
+    logger.info("[CV%s] creating %d models", prefix, len(params))
     for ident, param in enumerate(params):
         model = client.submit(_create_model, original_model, ident, **param)
         info[ident] = []
         models[ident] = model
+    for ident in info:
+        m, m2 = await models[ident].result()
 
     # assume everything in fit_params is small and make it concrete
-    fit_params = yield client.compute(fit_params)
+    fit_params = await client.compute(fit_params)
 
     # Convert testing data into a single element on the cluster
     # This assumes that it fits into memory on a single worker
+    if isinstance(X_train, (dd.DataFrame, dd.Series)):
+        X_train = X_train.to_dask_array()
+    if isinstance(X_test, (dd.DataFrame, dd.Series)):
+        X_test = X_test.to_dask_array()
+    if isinstance(y_train, dd.Series):
+        y_train = y_train.to_dask_array()
+    if isinstance(y_test, dd.Series):
+        y_test = y_test.to_dask_array()
+
+    X_train, y_train, X_test, y_test = dask.persist(X_train, y_train, X_test, y_test)
+
     if isinstance(X_test, da.Array):
         X_test = client.compute(X_test)
     else:
-        X_test = yield client.scatter(X_test)
+        X_test = await client.scatter(X_test)
     if isinstance(y_test, da.Array):
         y_test = client.compute(y_test)
     else:
-        y_test = yield client.scatter(y_test)
+        y_test = await client.scatter(y_test)
 
     # Convert to batches of delayed objects of numpy arrays
-    X_train, y_train = dask.persist(X_train, y_train)
     X_train = sorted(futures_of(X_train), key=lambda f: f.key)
     y_train = sorted(futures_of(y_train), key=lambda f: f.key)
     assert len(X_train) == len(y_train)
 
+    train_eg = await client.gather(client.map(len, y_train))
+    msg = "[CV%s] For training there are between %d and %d examples in each chunk"
+    logger.info(msg, prefix, min(train_eg), max(train_eg))
+
     # Order by which we process training data futures
     order = []
 
@@ -175,9 +229,9 @@ def _fit(
     # Submit initial partial_fit and score computations on first batch of data
     X_future, y_future = get_futures(0)
     X_future_2, y_future_2 = get_futures(1)
-    _models = {}
-    _scores = {}
-    _specs = {}
+    _models: Dict[int, Tuple[Model, Meta]] = {}
+    _scores: Dict[int, Meta] = {}
+    _specs: Dict[int, Tuple[Model, Meta]] = {}
 
     d_partial_fit = dask.delayed(_partial_fit)
     d_score = dask.delayed(_score)
@@ -204,8 +258,14 @@ def _fit(
     start_time = time()
 
     # async for future, result in seq:
-    while True:
-        metas = yield client.gather(new_scores)
+    for _i in itertools.count():
+        metas = await client.gather(new_scores)
+
+        if log_delay and _i % int(log_delay) == 0:
+            idx = np.argmax([m["score"] for m in metas])
+            best = metas[idx]
+            msg = "[CV%s] validation score of %0.4f received after %d partial_fit calls"
+            logger.info(msg, prefix, best["score"], best["partial_fit_calls"])
 
         for meta in metas:
             ident = meta["model_id"]
@@ -215,10 +275,10 @@ def _fit(
             history.append(meta)
 
         instructions = additional_calls(info)
-        bad = set(models) - set(instructions)
+        fired = set(models) - set(instructions)
 
-        # Delete the futures of bad models.  This cancels speculative tasks
-        for ident in bad:
+        # Delete the futures of bad/fired models.  This cancels speculative tasks
+        for ident in fired:
             del models[ident]
             del scores[ident]
             del info[ident]
@@ -232,6 +292,7 @@ def _fit(
 
         for ident, k in instructions.items():
             start = info[ident][-1]["partial_fit_calls"] + 1
+
             if k:
                 k -= 1
                 model = speculative.pop(ident)
@@ -262,8 +323,8 @@ def _fit(
         new_scores = list(_scores2.values())
 
     models = {k: client.submit(operator.getitem, v, 0) for k, v in models.items()}
-    yield wait(models)
-    scores = yield client.gather(scores)
+    await wait(models)
+    scores = await client.gather(scores)
     best = max(scores.items(), key=lambda x: x[1]["score"])
 
     info = defaultdict(list)
@@ -272,10 +333,10 @@ def _fit(
         info[h["model_id"]].append(h)
     info = dict(info)
 
-    raise gen.Return(Results(info, models, history, best))
+    return Results(info, models, history, best)
 
 
-def fit(
+async def fit(
     model,
     params,
     X_train,
@@ -286,6 +347,8 @@ def fit(
     fit_params=None,
     scorer=None,
     random_state=None,
+    verbose: Union[bool, int] = False,
+    prefix="",
 ):
     """ Find a good model and search among a space of hyper-parameters
 
@@ -325,6 +388,14 @@ def fit(
         If RandomState instance, random_state is the random number generator;
         If None, the random number generator is the RandomState instance used
         by `np.random`.
+    verbose : bool, int, float, default=False
+        If bool (default), log everytime possible.
+        If non-zero, configure logging to print/pipe to stdout.
+        If float or int, log and print ``verbose`` fraction of the time.
+        If zero, do not log past initialization.
+    prefix : str, optional, default: ""
+        The string to print out in each debug message. Each message is prefixed
+        with `[CV{prefix}]`.
 
     Examples
     --------
@@ -392,8 +463,7 @@ def fit(
     history : List[Dict]
         A history of all models scores over time
     """
-    return default_client().sync(
-        _fit,
+    return await _fit(
         model,
         params,
         X_train,
@@ -404,6 +474,8 @@ def fit(
         fit_params=fit_params,
         scorer=scorer,
         random_state=random_state,
+        verbose=verbose,
+        prefix=prefix,
     )
 
 
@@ -430,6 +502,8 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         max_iter=100,
         patience=False,
         tol=1e-3,
+        verbose=False,
+        prefix="",
     ):
         self.parameters = parameters
         self.test_size = test_size
@@ -437,6 +511,8 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         self.max_iter = max_iter
         self.patience = patience
         self.tol = tol
+        self.verbose = verbose
+        self.prefix = prefix
         super(BaseIncrementalSearchCV, self).__init__(estimator, scoring=scoring)
 
     def _validate_parameters(self, X, y):
@@ -447,8 +523,12 @@ class BaseIncrementalSearchCV(ParallelPostFit):
                 )
             )
 
-        X = self._check_array(X)
-        y = self._check_array(y, ensure_2d=False)
+        # Make sure dask arrays are passed so error on unknown chunk size is raised
+        kwargs = dict(accept_unknown_chunks=True, accept_dask_dataframe=True)
+        if not isinstance(X, dd.DataFrame):
+            X = self._check_array(X, **kwargs)
+        if not isinstance(y, dd.Series):
+            y = self._check_array(y, ensure_2d=False, **kwargs)
         scorer = check_scoring(self.estimator, scoring=self.scoring)
         return X, y, scorer
 
@@ -468,7 +548,6 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         """
         if isinstance(X, np.ndarray):
             X = da.from_array(X, X.shape)
-        X = check_array(X, **kwargs)
         return X
 
     def _get_train_test_split(self, X, y, **kwargs):
@@ -487,7 +566,7 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         else:
             test_size = self.test_size
         X_train, X_test, y_train, y_test = train_test_split(
-            X, y, test_size=test_size, random_state=self.random_state
+            X, y, test_size=test_size, random_state=self.random_state, shuffle=True
         )
         return X_train, X_test, y_train, y_test
 
@@ -501,16 +580,18 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         """
         return ParameterGrid(self.parameters)
 
-    def _get_cv_results(self, history, model_hist):
-        cv_results = {}
+    def _get_cv_results(
+        self, model_hist: Dict[int, List[Meta]]
+    ) -> Dict[str, List[Any]]:
+        _cv_results = {}
         best_scores = {}
         best_scores = {k: hist[-1]["score"] for k, hist in model_hist.items()}
 
-        cv_results = {}
+        _cv_results = {}
         for k, hist in model_hist.items():
             pf_times = list(toolz.pluck("partial_fit_time", hist))
             score_times = list(toolz.pluck("score_time", hist))
-            cv_results[k] = {
+            _cv_results[k] = {
                 "mean_partial_fit_time": np.mean(pf_times),
                 "mean_score_time": np.mean(score_times),
                 "std_partial_fit_time": np.std(pf_times),
@@ -520,8 +601,8 @@ class BaseIncrementalSearchCV(ParallelPostFit):
                 "params": hist[0]["params"],
                 "partial_fit_calls": hist[-1]["partial_fit_calls"],
             }
-        cv_results = list(cv_results.values())  # list of dicts
-        cv_results = {k: [res[k] for res in cv_results] for k in cv_results[0]}
+        _cv_results2: List[Dict[str, Any]] = list(_cv_results.values())
+        cv_results = {k: [res[k] for res in _cv_results2] for k in _cv_results2[0]}
 
         # Every model will have the same params because this class uses either
         # ParameterSampler or ParameterGrid
@@ -539,7 +620,7 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         ).astype(int)
         return cv_results
 
-    def _process_results(self, results):
+    def _process_results(self, results: Results):
         """Called with the output of `fit` immediately after it finishes.
 
         Subclasses may update the results here, before further results are
@@ -552,29 +633,38 @@ class BaseIncrementalSearchCV(ParallelPostFit):
     def _check_is_fitted(self, method_name):
         return check_is_fitted(self, "best_estimator_")
 
-    @gen.coroutine
-    def _fit(self, X, y, **fit_params):
+    async def _fit(self, X, y, **fit_params):
+        if self.verbose:
+            h = logging.StreamHandler(sys.stdout)
+            context = LoggingContext(logger, level=logging.INFO, handler=h)
+        else:
+            context = dummy_context()
+
         X, y, scorer = self._validate_parameters(X, y)
+
         X_train, X_test, y_train, y_test = self._get_train_test_split(X, y)
 
-        results = yield fit(
-            self.estimator,
-            self._get_params(),
-            X_train,
-            y_train,
-            X_test,
-            y_test,
-            additional_calls=self._additional_calls,
-            fit_params=fit_params,
-            scorer=scorer,
-            random_state=self.random_state,
-        )
+        with context:
+            results = await fit(
+                self.estimator,
+                self._get_params(),
+                X_train,
+                y_train,
+                X_test,
+                y_test,
+                additional_calls=self._additional_calls,
+                fit_params=fit_params,
+                scorer=scorer,
+                random_state=self.random_state,
+                verbose=self.verbose,
+                prefix=self.prefix,
+            )
         results = self._process_results(results)
         model_history, models, history, bst = results
 
-        cv_results = self._get_cv_results(history, model_history)
+        cv_results = self._get_cv_results(model_history)
         best_idx = bst[0]
-        best_estimator = yield models[best_idx]
+        best_estimator = await models[best_idx]
 
         # Clean up models we're hanging onto
         ids = list(results.models)
@@ -596,7 +686,7 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         # at each scoring, but one score is needed to choose the better of two
         # models
         self.multimetric_ = False
-        raise gen.Return(self)
+        return self
 
     def fit(self, X, y=None, **fit_params):
         """Find the best parameters for a particular model.
@@ -607,7 +697,10 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         **fit_params
             Additional partial fit keyword arguments for the estimator.
         """
-        return default_client().sync(self._fit, X, y, **fit_params)
+        client = default_client()
+        if not client.asynchronous:
+            return client.sync(self._fit, X, y, **fit_params)
+        return self._fit(X, y, **fit_params)
 
     @if_delegate_has_method(delegate=("best_estimator_", "estimator"))
     def decision_function(self, X):
@@ -624,7 +717,7 @@ class BaseIncrementalSearchCV(ParallelPostFit):
         self._check_is_fitted("inverse_transform")
         return self.best_estimator_.transform(Xt)
 
-    def score(self, X, y=None):
+    def score(self, X, y=None) -> float:
         if self.scorer_ is None:
             raise ValueError(
                 "No score function explicitly defined, "
@@ -641,12 +734,6 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
     model on many hyper-parameters on a small amount of data, and then only
     continues training those models that seem to be performing well.
 
-    The number of actively trained hyper-parameter combinations decays
-    with an inverse decay given by the initial number of parameters and the
-    decay rate:
-
-        n_models = n_initial_parameters * (n_batches ** -decay_rate)
-
     See the :ref:`User Guide <hyperparameter.incremental>` for more.
 
     Parameters
@@ -672,17 +759,27 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
 
     decay_rate : float, default 1.0
         How quickly to decrease the number partial future fit calls.
-        Higher `decay_rate` will result in lower training times, at the cost
-        of worse models.
+
+        .. deprecated:: v1.4.0
+           This implementation of an adaptive algorithm that uses
+           ``decay_rate`` has moved to
+           :class:`~dask_ml.model_selection.InverseDecaySearchCV`.
 
     patience : int, default False
         If specified, training stops when the score does not increase by
         ``tol`` after ``patience`` calls to ``partial_fit``. Off by default.
 
+    fits_per_score : int, optional, default=1
+        If ``patience`` is used the maximum number of ``partial_fit`` calls
+        between ``score`` calls.
+
     scores_per_fit : int, default 1
         If ``patience`` is used the maximum number of ``partial_fit`` calls
         between ``score`` calls.
 
+        .. deprecated:: v1.4.0
+           Renamed to ``fits_per_score``.
+
     tol : float, default 0.001
         The required level of improvement to consider stopping training on
         that model. The most recent score must be at at most ``tol`` better
@@ -723,6 +820,17 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
 
         If None, the estimator's default scorer (if available) is used.
 
+    verbose : bool, float, int, optional, default: False
+        If False (default), don't print logs (or pipe them to stdout). However,
+        standard logging will still be used.
+
+        If True, print logs and use standard logging.
+
+        If float, print/log approximately ``verbose`` fraction of the time.
+
+    prefix : str, optional, default=""
+        While logging, add ``prefix`` to each message.
+
     Attributes
     ----------
     cv_results_ : dict of np.ndarrays
@@ -835,6 +943,7 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
     For example, setting ``tol=0`` and ``patience=2`` means training will stop
     after two consecutive calls to ``model.partial_fit`` without improvement,
     or when ``max_iter`` total calls to ``model.parital_fit`` are reached.
+
     """
 
     def __init__(
@@ -842,21 +951,24 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
         estimator,
         parameters,
         n_initial_parameters=10,
-        decay_rate=1.0,
+        decay_rate=no_default,
         test_size=None,
         patience=False,
         tol=0.001,
-        scores_per_fit=1,
+        fits_per_score=1,
         max_iter=100,
         random_state=None,
         scoring=None,
+        verbose=False,
+        prefix="",
+        scores_per_fit=None,
     ):
+
         self.n_initial_parameters = n_initial_parameters
         self.decay_rate = decay_rate
-        self.patience = patience
-        self.tol = tol
+        self.fits_per_score = fits_per_score
         self.scores_per_fit = scores_per_fit
-        self.max_iter = max_iter
+
         super(IncrementalSearchCV, self).__init__(
             estimator,
             parameters,
@@ -866,8 +978,40 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
             max_iter=max_iter,
             patience=patience,
             tol=tol,
+            verbose=verbose,
+            prefix=prefix,
         )
 
+    def _decay_deprecated(self):
+        return True
+
+    def fit(self, X, y=None, **fit_params):
+        if self._decay_deprecated():
+            if self.decay_rate is no_default:
+                warn(
+                    "decay_rate has been deprecated since Dask-ML v1.4.0.\n\n"
+                    "    * Use InverseDecaySearchCV to use `decay_rate`\n"
+                    "    * Specify decay_rate=None\n\n",
+                    FutureWarning,
+                )
+            elif self.decay_rate is not None:
+                warn(
+                    "decay_rate is deprecated in InverseDecaySearchCV. "
+                    f"Use InverseDecaySearchCV to use decay_rate={self.decay_rate}",
+                    FutureWarning,
+                )
+        if self.scores_per_fit is not None and self.fits_per_score != 1:
+            msg = "Specify fits_per_score, not scores_per_fit"
+            raise ValueError(msg)
+
+        if self.scores_per_fit:
+            self.fits_per_score = self.scores_per_fit
+            warn(
+                "scores_per_fit has been deprecated since Dask-ML v1.4.0. "
+                "Specify fits_per_score={} instead".format(self.scores_per_fit)
+            )
+        return super(IncrementalSearchCV, self).fit(X, y=y, **fit_params)
+
     def _get_params(self):
         if self.n_initial_parameters == "grid":
             return ParameterGrid(self.parameters)
@@ -878,17 +1022,17 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
                 random_state=self.random_state,
             )
 
-    def _additional_calls(self, info):
+    def _additional_calls(self, info: Dict[int, List[Meta]]) -> Instructions:
         if not isinstance(self.patience, int):
             msg = (
                 "patience must be an integer (or a subclass like boolean), "
                 "not patience={} of type {}"
             )
             raise ValueError(msg.format(self.patience, type(self.patience)))
-        if not isinstance(self.patience, bool) and self.patience <= 1:
+        if self.patience and self.patience <= 1:  # patience=0 => don't use patience
             raise ValueError(
                 "patience={}<=1 will always detect a plateau. "
-                "this, set\n\n    patience >= 2"
+                "To resolve this,\n\n    * set patience >= 2"
             )
 
         calls = {k: v[-1]["partial_fit_calls"] for k, v in info.items()}
@@ -919,42 +1063,7 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
         return out
 
     def _adapt(self, info):
-        # First, have an adaptive algorithm
-        if self.n_initial_parameters == "grid":
-            start = len(ParameterGrid(self.parameters))
-        else:
-            start = self.n_initial_parameters
-
-        def inverse(time):
-            """ Decrease target number of models inversely with time """
-            return int(start / (1 + time) ** self.decay_rate)
-
-        example = toolz.first(info.values())
-        time_step = example[-1]["partial_fit_calls"]
-
-        current_time_step = time_step + 1
-        next_time_step = current_time_step
-
-        if inverse(current_time_step) == 0:
-            # we'll never get out of here
-            next_time_step = 1
-
-        while inverse(current_time_step) == inverse(next_time_step) and (
-            self.decay_rate
-            and not self.patience
-            or next_time_step - current_time_step < self.scores_per_fit
-        ):
-            next_time_step += 1
-
-        target = max(1, inverse(next_time_step))
-        best = toolz.topk(target, info, key=lambda k: info[k][-1]["score"])
-
-        if len(best) == 1:
-            [best] = best
-            return {best: 0}
-        steps = next_time_step - current_time_step
-        instructions = {b: steps for b in best}
-        return instructions
+        return {k: self.fits_per_score for k in info}
 
     def _stop_on_plateau(self, instructions, info):
         # Second, stop on plateau if any models have already converged
@@ -979,3 +1088,267 @@ class IncrementalSearchCV(BaseIncrementalSearchCV):
             else:
                 out[k] = steps
         return out
+
+
+class InverseDecaySearchCV(IncrementalSearchCV):
+    """
+    Incrementally search for hyper-parameters on models that support partial_fit
+
+    This incremental hyper-parameter optimization class starts training the
+    model on many hyper-parameters on a small amount of data, and then only
+    continues training those models that seem to be performing well.
+
+    This class will decay the number of parameters over time. At time step
+    ``k``, this class will retain ``1 / (k + 1)`` fraction of the highest
+    performing models.
+
+    Parameters
+    ----------
+    estimator : estimator object.
+        A object of that type is instantiated for each initial hyperparameter
+        combination. This is assumed to implement the scikit-learn estimator
+        interface. Either estimator needs to provide a `score`` function,
+        or ``scoring`` must be passed. The estimator must implement
+        ``partial_fit``, ``set_params``, and work well with ``clone``.
+
+    parameters : dict
+        Dictionary with parameters names (string) as keys and distributions
+        or lists of parameters to try. Distributions must provide a ``rvs``
+        method for sampling (such as those from scipy.stats.distributions).
+        If a list is given, it is sampled uniformly.
+
+    n_initial_parameters : int, default=10
+        Number of parameter settings that are sampled.
+        This trades off runtime vs quality of the solution.
+
+        Alternatively, you can set this to ``"grid"`` to do a full grid search.
+
+    patience : int, default False
+        If specified, training stops when the score does not increase by
+        ``tol`` after ``patience`` calls to ``partial_fit``. Off by default.
+
+    fits_per_scores : int, optional, default=1
+        If ``patience`` is used the maximum number of ``partial_fit`` calls
+        between ``score`` calls.
+
+    scores_per_fit : int, default 1
+        If ``patience`` is used the maximum number of ``partial_fit`` calls
+        between ``score`` calls.
+
+    tol : float, default 0.001
+        The required level of improvement to consider stopping training on
+        that model. The most recent score must be at at most ``tol`` better
+        than the all of the previous ``patience`` scores for that model.
+        Increasing ``tol`` will tend to reduce training time, at the cost
+        of worse models.
+
+    max_iter : int, default 100
+        Maximum number of partial fit calls per model.
+
+    test_size : float
+        Fraction of the dataset to hold out for computing test scores.
+        Defaults to the size of a single partition of the input training set
+
+        .. note::
+
+           The training dataset should fit in memory on a single machine.
+           Adjust the ``test_size`` parameter as necessary to achieve this.
+
+    random_state : int, RandomState instance or None, optional, default: None
+        If int, random_state is the seed used by the random number generator;
+        If RandomState instance, random_state is the random number generator;
+        If None, the random number generator is the RandomState instance used
+        by `np.random`.
+
+    scoring : string, callable, list/tuple, dict or None, default: None
+        A single string (see :ref:`scoring_parameter`) or a callable
+        (see :ref:`scoring`) to evaluate the predictions on the test set.
+
+        For evaluating multiple metrics, either give a list of (unique) strings
+        or a dict with names as keys and callables as values.
+
+        NOTE that when using custom scorers, each scorer should return a single
+        value. Metric functions returning a list/array of values can be wrapped
+        into multiple scorers that return one value each.
+
+        See :ref:`multimetric_grid_search` for an example.
+
+        If None, the estimator's default scorer (if available) is used.
+
+    verbose : bool, float, int, optional, default: False
+        If False (default), don't print logs (or pipe them to stdout). However,
+        standard logging will still be used.
+
+        If True, print logs and use standard logging.
+
+        If float, print/log approximately ``verbose`` fraction of the time.
+
+    prefix : str, optional, default=""
+        While logging, add ``prefix`` to each message.
+
+    decay_rate : float, default 1.0
+        How quickly to decrease the number partial future fit calls.
+        Higher `decay_rate` will result in lower training times, at the cost
+        of worse models.
+
+        The default ``decay_rate=1.0`` is chosen because it has some theoritical
+        motivation [1]_.
+
+    Attributes
+    ----------
+    cv_results_ : dict of np.ndarrays
+        This dictionary has keys
+
+        * ``mean_partial_fit_time``
+        * ``mean_score_time``
+        * ``std_partial_fit_time``
+        * ``std_score_time``
+        * ``test_score``
+        * ``rank_test_score``
+        * ``model_id``
+        * ``partial_fit_calls``
+        * ``params``
+        * ``param_{key}``, where ``key`` is every key in ``params``.
+
+        The values in the ``test_score`` key correspond to the last score a model
+        received on the hold out dataset. The key ``model_id`` corresponds with
+        ``history_``. This dictionary can be imported into Pandas.
+
+    model_history_ : dict of lists of dict
+        A dictionary of each models history. This is a reorganization of
+        ``history_``: the same information is present but organized per model.
+
+        This data has the structure  ``{model_id: hist}`` where ``hist`` is a
+        subset of ``history_`` and ``model_id`` are model identifiers.
+
+    history_ : list of dicts
+        Information about each model after each ``partial_fit`` call. Each dict
+        the keys
+
+        * ``partial_fit_time``
+        * ``score_time``
+        * ``score``
+        * ``model_id``
+        * ``params``
+        * ``partial_fit_calls``
+        * ``elapsed_wall_time``
+
+        The key ``model_id`` corresponds to the ``model_id`` in ``cv_results_``.
+        This list of dicts can be imported into Pandas.
+
+    best_estimator_ : BaseEstimator
+        The model with the highest validation score among all the models
+        retained by the "inverse decay" algorithm.
+
+    best_score_ : float
+        Score achieved by ``best_estimator_`` on the vaidation set after the
+        final call to ``partial_fit``.
+
+    best_index_ : int
+        Index indicating which estimator in ``cv_results_`` corresponds to
+        the highest score.
+
+    best_params_ : dict
+        Dictionary of best parameters found on the hold-out data.
+
+    scorer_ :
+        The function used to score models, which has a call signature of
+        ``scorer_(estimator, X, y)``.
+
+    n_splits_ : int
+        Number of cross validation splits.
+
+    multimetric_ : bool
+        Whether this cross validation search uses multiple metrics.
+
+    Notes
+    -----
+    When ``decay_rate==1``, this class approximates the
+    number of ``partial_fit`` calls that :class:`SuccesiveHalvingSearchCV`
+    performs. If ``n_initial_parameters`` is configured properly with
+    ``decay_rate=1``, it's possible this class will mirror the most aggressive
+    bracket of :class:`HyperbandSearchCV`. This might yield good results
+    and/or find good models, but is untested.
+
+    References
+    ----------
+    .. [1] Li, L., Jamieson, K., DeSalvo, G., Rostamizadeh, A., & Talwalkar, A.
+           (2017). Hyperband: A novel bandit-based approach to hyperparameter
+           optimization. The Journal of Machine Learning Research, 18(1),
+           6765-6816. http://www.jmlr.org/papers/volume18/16-558/16-558.pdf
+
+    """
+
+    def __init__(
+        self,
+        estimator,
+        parameters,
+        n_initial_parameters=10,
+        test_size=None,
+        patience=False,
+        tol=0.001,
+        fits_per_score=1,
+        max_iter=100,
+        random_state=None,
+        scoring=None,
+        verbose=False,
+        prefix="",
+        decay_rate=1.0,
+    ):
+        self.decay_rate = decay_rate
+        super(InverseDecaySearchCV, self).__init__(
+            estimator,
+            parameters,
+            n_initial_parameters=n_initial_parameters,
+            test_size=test_size,
+            patience=patience,
+            tol=tol,
+            fits_per_score=fits_per_score,
+            max_iter=max_iter,
+            random_state=random_state,
+            scoring=scoring,
+            verbose=verbose,
+            prefix=prefix,
+            decay_rate=decay_rate,
+        )
+
+    def _decay_deprecated(self):
+        return False
+
+    def _adapt(self, info):
+        # First, have an adaptive algorithm
+        if self.n_initial_parameters == "grid":
+            start = len(ParameterGrid(self.parameters))
+        else:
+            start = self.n_initial_parameters
+
+        def inverse(time):
+            """ Decrease target number of models inversely with time """
+            return int(start / (1 + time) ** self.decay_rate)
+
+        example = toolz.first(info.values())
+        time_step = example[-1]["partial_fit_calls"]
+
+        current_time_step = time_step + 1
+        next_time_step = current_time_step
+
+        if inverse(current_time_step) == 0:
+            # we'll never get out of here
+            next_time_step = 1
+
+        while inverse(current_time_step) == inverse(next_time_step) and (
+            self.decay_rate
+            and not self.patience
+            or next_time_step - current_time_step < self.fits_per_score
+        ):
+            next_time_step += 1
+
+        target = max(1, inverse(next_time_step))
+        best = toolz.topk(target, info, key=lambda k: info[k][-1]["score"])
+
+        if len(best) == 1:
+            [best] = best
+            return {best: 0}
+        steps = next_time_step - current_time_step
+        instructions = {b: steps for b in best}
+        return instructions
