@@ -3,6 +3,7 @@ from __future__ import division
 import multiprocessing
 from collections import OrderedDict
 from distutils.version import LooseVersion
+from typing import Any, List, Optional, Sequence, Union
 
 import dask.array as da
 import dask.dataframe as dd
@@ -16,16 +17,18 @@ from scipy import stats
 from sklearn.base import BaseEstimator, TransformerMixin
 from sklearn.utils.validation import check_random_state
 
-from dask_ml._compat import SK_022, blockwise, check_is_fitted
+from dask_ml._compat import blockwise, check_is_fitted
 from dask_ml._utils import copy_learned_attributes
 from dask_ml.utils import check_array, handle_zeros_in_scale
 
+from .._typing import ArrayLike, DataFrameType, SeriesType
+
 _PANDAS_VERSION = LooseVersion(pd.__version__)
 _HAS_CTD = _PANDAS_VERSION >= "0.21.0"
 BOUNDS_THRESHOLD = 1e-7
 
 
-def _handle_zeros_in_scale(scale, copy=True):
+def _handle_zeros_in_scale(scale: np.ndarray, copy=True):
     """ Makes sure that whenever scale is zero, we handle it correctly.
 
     This happens in most scalers when we have constant features."""
@@ -47,7 +50,11 @@ class StandardScaler(sklearn.preprocessing.StandardScaler):
 
     __doc__ = sklearn.preprocessing.StandardScaler.__doc__
 
-    def fit(self, X, y=None):
+    def fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ) -> "StandardScaler":
         self._reset()
         attributes = OrderedDict()
         if isinstance(X, (pd.DataFrame, dd.DataFrame)):
@@ -68,19 +75,31 @@ class StandardScaler(sklearn.preprocessing.StandardScaler):
         values = compute(*attributes.values())
         for k, v in zip(attributes, values):
             setattr(self, k, v)
+        self.n_features_in_ = X.shape[1]
         return self
 
-    def partial_fit(self, X, y=None):
+    def partial_fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ):
         raise NotImplementedError()
 
-    def transform(self, X, y=None, copy=None):
+    def transform(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+        copy: Optional[bool] = None,
+    ) -> Union[ArrayLike, DataFrameType]:
         if self.with_mean:
             X -= self.mean_
         if self.with_std:
             X /= self.scale_
         return X
 
-    def inverse_transform(self, X, copy=None):
+    def inverse_transform(
+        self, X: Union[ArrayLike, DataFrameType], copy: Optional[bool] = None
+    ) -> Union[ArrayLike, DataFrameType]:
         if self.with_std:
             X *= self.scale_
         if self.with_mean:
@@ -92,7 +111,11 @@ class MinMaxScaler(sklearn.preprocessing.MinMaxScaler):
 
     __doc__ = sklearn.preprocessing.MinMaxScaler.__doc__
 
-    def fit(self, X, y=None):
+    def fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ) -> "MinMaxScaler":
         self._reset()
         attributes = OrderedDict()
         feature_range = self.feature_range
@@ -119,12 +142,22 @@ class MinMaxScaler(sklearn.preprocessing.MinMaxScaler):
         values = compute(*attributes.values())
         for k, v in zip(attributes, values):
             setattr(self, k, v)
+        self.n_features_in_ = X.shape[1]
         return self
 
-    def partial_fit(self, X, y=None):
+    def partial_fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ):
         raise NotImplementedError()
 
-    def transform(self, X, y=None, copy=None):
+    def transform(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+        copy: Optional[bool] = None,
+    ) -> Union[ArrayLike, DataFrameType]:
         # Workaround for https://github.com/dask/dask/issues/2840
         if isinstance(X, dd.DataFrame):
             X = X.mul(self.scale_).add(self.min_)
@@ -133,7 +166,12 @@ class MinMaxScaler(sklearn.preprocessing.MinMaxScaler):
             X = X + self.min_
         return X
 
-    def inverse_transform(self, X, y=None, copy=None):
+    def inverse_transform(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+        copy: Optional[bool] = None,
+    ) -> Union[ArrayLike, DataFrameType]:
         if not hasattr(self, "scale_"):
             raise Exception(
                 "This %(name)s instance is not fitted yet. "
@@ -155,11 +193,17 @@ class RobustScaler(sklearn.preprocessing.RobustScaler):
 
     __doc__ = sklearn.preprocessing.RobustScaler.__doc__
 
-    def _check_array(self, X, *args, **kwargs):
+    def _check_array(
+        self, X: Union[ArrayLike, DataFrameType], *args: Any, **kwargs: Any
+    ) -> Union[ArrayLike, DataFrameType]:
         X = check_array(X, accept_dask_dataframe=True, **kwargs)
         return X
 
-    def fit(self, X, y=None):
+    def fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ) -> "RobustScaler":
         q_min, q_max = self.quantile_range
         if not 0 <= q_min <= q_max <= 100:
             raise ValueError("Invalid quantile range: %s" % str(self.quantile_range))
@@ -178,14 +222,17 @@ class RobustScaler(sklearn.preprocessing.RobustScaler):
                 ]
             )
 
-        quantiles = [da.percentile(col, [q_min, 50.0, q_max]) for col in X.T]
+        quantiles: Any = [da.percentile(col, [q_min, 50.0, q_max]) for col in X.T]
         quantiles = da.vstack(quantiles).compute()
-        self.center_ = quantiles[:, 1]
-        self.scale_ = quantiles[:, 2] - quantiles[:, 0]
+        self.center_: List[float] = quantiles[:, 1]
+        self.scale_: List[float] = quantiles[:, 2] - quantiles[:, 0]
         self.scale_ = _handle_zeros_in_scale(self.scale_, copy=False)
+        self.n_features_in_ = X.shape[1]
         return self
 
-    def transform(self, X):
+    def transform(
+        self, X: Union[ArrayLike, DataFrameType]
+    ) -> Union[ArrayLike, DataFrameType]:
         """Center and scale the data.
 
         Can be called on sparse input, provided that ``RobustScaler`` has been
@@ -217,7 +264,9 @@ class RobustScaler(sklearn.preprocessing.RobustScaler):
             X /= self.scale_
         return X
 
-    def inverse_transform(self, X):
+    def inverse_transform(
+        self, X: Union[ArrayLike, DataFrameType]
+    ) -> Union[ArrayLike, DataFrameType]:
         """Scale back the data to the original representation
 
         Parameters
@@ -254,11 +303,13 @@ class QuantileTransformer(sklearn.preprocessing.QuantileTransformer):
         sklearn.preprocessing.QuantileTransformer.__doc__.split("\n")[1:]
     )
 
-    def _check_inputs(self, X, accept_sparse_negative=False, copy=False):
-        kwargs = {}
-        if SK_022:
-            kwargs["copy"] = copy
-
+    def _check_inputs(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        accept_sparse_negative: bool = False,
+        copy: bool = False,
+        in_fit: bool = True,
+    ) -> Union[ArrayLike, DataFrameType]:
         if isinstance(X, (pd.DataFrame, dd.DataFrame)):
             X = X.values
         if isinstance(X, np.ndarray):
@@ -271,19 +322,26 @@ class QuantileTransformer(sklearn.preprocessing.QuantileTransformer):
         # TODO: mix of sparse, dense?
         sample = rng.uniform(size=(5, X.shape[1])).astype(X.dtype)
         super(QuantileTransformer, self)._check_inputs(
-            sample, accept_sparse_negative=accept_sparse_negative, **kwargs
+            sample,
+            accept_sparse_negative=accept_sparse_negative,
+            copy=copy,
+            in_fit=in_fit,
         )
         return X
 
-    def _sparse_fit(self, X, random_state):
+    def _sparse_fit(self, X: Union[ArrayLike, DataFrameType], random_state: int):
         raise NotImplementedError
 
-    def _dense_fit(self, X, random_state):
+    def _dense_fit(
+        self, X: Union[ArrayLike, DataFrameType], random_state: int
+    ) -> Union[ArrayLike, DataFrameType]:
         references = self.references_ * 100
         quantiles = [da.percentile(col, references) for col in X.T]
-        self.quantiles_, = compute(da.vstack(quantiles).T)
+        (self.quantiles_,) = compute(da.vstack(quantiles).T)
 
-    def _transform(self, X, inverse=False):
+    def _transform(
+        self, X: Union[ArrayLike, DataFrameType], inverse: bool = False
+    ) -> Union[ArrayLike, DataFrameType]:
         X = X.copy()  # ...
         transformed = [
             self._transform_col(
@@ -293,7 +351,9 @@ class QuantileTransformer(sklearn.preprocessing.QuantileTransformer):
         ]
         return da.vstack(transformed, allow_unknown_chunksizes=True).T
 
-    def _transform_col(self, X_col, quantiles, inverse):
+    def _transform_col(
+        self, X_col: ArrayLike, quantiles: ArrayLike, inverse: bool
+    ) -> ArrayLike:
         output_distribution = self.output_distribution
 
         if not inverse:
@@ -412,11 +472,11 @@ class Categorizer(BaseEstimator, TransformerMixin):
     CategoricalDtype(categories=['a', 'b', 'c'], ordered=False)
     """
 
-    def __init__(self, categories=None, columns=None):
+    def __init__(self, categories: Optional[dict] = None, columns: pd.Index = None):
         self.categories = categories
         self.columns = columns
 
-    def _check_array(self, X):
+    def _check_array(self, X: DataFrameType) -> DataFrameType:
         # TODO: refactor to check_array
         if not isinstance(X, (pd.DataFrame, dd.DataFrame)):
             raise TypeError(
@@ -424,7 +484,9 @@ class Categorizer(BaseEstimator, TransformerMixin):
             )
         return X
 
-    def fit(self, X, y=None):
+    def fit(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> "Categorizer":
         """Find the categorical columns.
 
         Parameters
@@ -452,7 +514,7 @@ class Categorizer(BaseEstimator, TransformerMixin):
         self.categories_ = categories
         return self
 
-    def _fit(self, X):
+    def _fit(self, X: DataFrameType):
         if self.columns is None:
             columns = X.select_dtypes(include=["object", "category"]).columns
         else:
@@ -473,12 +535,14 @@ class Categorizer(BaseEstimator, TransformerMixin):
 
         return columns, categories
 
-    def _fit_dask(self, X):
+    def _fit_dask(self, X: DataFrameType):
         columns = self.columns
         df = X.categorize(columns=columns, index=False)
         return self._fit(df)
 
-    def transform(self, X, y=None):
+    def transform(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> DataFrameType:
         """Transform the columns in ``X`` according to ``self.categories_``.
 
         Parameters
@@ -589,11 +653,15 @@ class DummyEncoder(BaseEstimator, TransformerMixin):
     Dask Name: get_dummies, 4 tasks
     """
 
-    def __init__(self, columns=None, drop_first=False):
+    def __init__(
+        self, columns: Optional[Sequence[Any]] = None, drop_first: bool = False
+    ):
         self.columns = columns
         self.drop_first = drop_first
 
-    def fit(self, X, y=None):
+    def fit(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> "DummyEncoder":
         """Determine the categorical columns to be dummy encoded.
 
         Parameters
@@ -642,7 +710,9 @@ class DummyEncoder(BaseEstimator, TransformerMixin):
         ).columns
         return self
 
-    def transform(self, X, y=None):
+    def transform(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> DataFrameType:
         """Dummy encode the categorical columns in X
 
         Parameters
@@ -667,7 +737,7 @@ class DummyEncoder(BaseEstimator, TransformerMixin):
         else:
             raise TypeError("Unexpected type {}".format(type(X)))
 
-    def inverse_transform(self, X):
+    def inverse_transform(self, X: Union[ArrayLike, DataFrameType]) -> DataFrameType:
         """Inverse dummy-encode the columns in `X`
 
         Parameters
@@ -691,7 +761,7 @@ class DummyEncoder(BaseEstimator, TransformerMixin):
             if unknown:
                 lengths = blockwise(len, "i", X[:, 0], "i", dtype="i8").compute()
                 X = X.copy()
-                chunks = (tuple(lengths), X.chunks[1])
+                chunks: ArrayLike = (tuple(lengths), X.chunks[1])
                 X._chunks = chunks
 
             X = dd.from_dask_array(X, columns=self.transformed_columns_)
@@ -827,7 +897,9 @@ class OrdinalEncoder(BaseEstimator, TransformerMixin):
     def __init__(self, columns=None):
         self.columns = columns
 
-    def fit(self, X, y=None):
+    def fit(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> "OrdinalEncoder":
         """Determine the categorical columns to be encoded.
 
         Parameters
@@ -860,7 +932,9 @@ class OrdinalEncoder(BaseEstimator, TransformerMixin):
 
         return self
 
-    def transform(self, X, y=None):
+    def transform(
+        self, X: DataFrameType, y: Optional[Union[ArrayLike, SeriesType]] = None
+    ) -> DataFrameType:
         """Ordinal encode the categorical columns in X
 
         Parameters
@@ -886,7 +960,9 @@ class OrdinalEncoder(BaseEstimator, TransformerMixin):
             X[col] = X[col].cat.codes
         return X
 
-    def inverse_transform(self, X):
+    def inverse_transform(
+        self, X: Union[ArrayLike, DataFrameType]
+    ) -> Union[ArrayLike, DataFrameType]:
         """Inverse ordinal-encode the columns in `X`
 
         Parameters
@@ -910,7 +986,7 @@ class OrdinalEncoder(BaseEstimator, TransformerMixin):
             if unknown:
                 lengths = blockwise(len, "i", X[:, 0], "i", dtype="i8").compute()
                 X = X.copy()
-                chunks = (tuple(lengths), X.chunks[1])
+                chunks: ArrayLike = (tuple(lengths), X.chunks[1])
                 X._chunks = chunks
 
             X = dd.from_dask_array(X, columns=self.columns_)
@@ -975,15 +1051,21 @@ class PolynomialFeatures(sklearn.preprocessing.PolynomialFeatures):
 
     def __init__(
         self,
-        degree=2,
-        interaction_only=False,
-        include_bias=True,
-        preserve_dataframe=False,
+        degree: int = 2,
+        interaction_only: bool = False,
+        include_bias: bool = True,
+        preserve_dataframe: bool = False,
     ):
-        super(PolynomialFeatures, self).__init__(degree, interaction_only, include_bias)
+        super(PolynomialFeatures, self).__init__(
+            degree=degree, interaction_only=interaction_only, include_bias=include_bias
+        )
         self.preserve_dataframe = preserve_dataframe
 
-    def fit(self, X, y=None):
+    def fit(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ) -> "PolynomialFeatures":
         self._transformer = sklearn.preprocessing.PolynomialFeatures(
             degree=self.degree,
             interaction_only=self.interaction_only,
@@ -1000,7 +1082,11 @@ class PolynomialFeatures(sklearn.preprocessing.PolynomialFeatures):
         copy_learned_attributes(self._transformer, self)
         return self
 
-    def transform(self, X, y=None):
+    def transform(
+        self,
+        X: Union[ArrayLike, DataFrameType],
+        y: Optional[Union[ArrayLike, SeriesType]] = None,
+    ) -> Union[ArrayLike, DataFrameType]:
         if isinstance(X, da.Array):
             n_cols = len(self._transformer.get_feature_names())
             X = check_array(X, accept_multiple_blocks=False, accept_unknown_chunks=True)
